#pragma once

#include <l4/sys/ipc.h>

#include "accessing.h"
#include "file_pager.h"
#include "opener_context_resource.h"
#include "opener_interface.h"
#include "pager.h"
#include "paging.h"
#include "pages.h"
#include "resource.h"

/* Support for providing access to files. */

class OpenerResource : public Resource, public Opener
{
protected:
    Accessing *_accessing;
    Paging *_paging;
    Pages *_pages;

    /* Convenience methods. */

    Accessor *get_accessor(fileid_t fileid);

    PageMapper *get_mapper(Accessor *accessor);

    /* Configurable methods. */

    virtual fileid_t get_fileid(const char *path) = 0;

    virtual Accessor *make_accessor(fileid_t fileid) = 0;

public:
    explicit OpenerResource(Accessing *accessing, Paging *paging, Pages *pages);

    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<Opener *>(this); }

    /* Direct access methods. */

    FilePager *open(const char *path, flags_t flags);

    /* Opener interface methods. */

    long context(l4_cap_idx_t *context);
};

// vim: tabstop=4 expandtab shiftwidth=4
